#!/usr/bin/env python
# -*- coding: utf-8 -*-

from base64 import b64encode as to64, b64decode as from64
import rsa

class NExchange:
    def __init__(self, key_size):
        _public, self._private = rsa.newkeys(key_size)
        self.public = '###START-OF-NIMING-RSA-PUBLIC###%s@@%s###END-OF-NIMING-RSA-PUBLIC###' % (_public.n, _public.e)

    def encrypt_key(self, key, public_key):
        splitted_key = public_key[32:-30].split("@@")
        public = rsa.PublicKey(int(splitted_key[0]), int(splitted_key[1]))
        return '###START-OF-NIMING-KEY###%s###END-OF-NIMING-KEY###' % to64(rsa.encrypt(key.encode(), public)).decode()

    def decrypt_key(self, encrypted_key):
        return rsa.decrypt(from64(encrypted_key[25:-23].encode()), self._private).decode()

    def identity_proof(self, username):
        return '###NIMING-IDENTITY-PROOF###%s###END-IDENTITY-PROOF###' % to64(rsa.sign(username.encode(), self._private, 'SHA-1')).decode()

    def verify_identity(self, proof, emitter_name, emitter_public_key):
        splitted_key = emitter_public_key[32:-30].split("@@")
        emit_public = rsa.PublicKey(int(splitted_key[0]), int(splitted_key[1]))
        proof = from64(proof[27:-24].encode())
        return rsa.verify(emitter_name.encode(), proof, emit_public)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import hashlib          #https://docs.python.org/fr/3.6/library/hashlib.html
import lzma             #utilitaire de compression de données basé sur l'algorithme LZMA
from NimingCypher.BaseConverter import IntToBase, BaseToInt
from secrets import choice #random but securly

__all__ = []
threshold = 10  #utilisé pour définir le nb d'occurrence minimale nécessaire pour l'utilisation dans la base
chunk_size = 64 #utilisé pour définir la taille des coupes de fichiers

def Intersperse(lst, item):
    result = [item] * (len(lst) * 2 - 1)
    result[0::2] = lst
    return result

def Chiffrer(clear_tx, listes_globales=[]):
    if listes_globales==[]:
        clear_tx, listes_globales = clear_tx
    liste_char, liste_pos = listes_globales

    good_ch=[hex(int(choice(liste_pos[liste_char.index(ch)])))[2:] for ch in clear_tx]

    final_gen=Intersperse(good_ch,' ')
    return ''.join(final_gen)

def Dechiffrer(encrypted_tx, TextFromWeb):
    out = []
    error = False
    for ch in encrypted_tx.split(' '): #pour chaque bloc hexa
        if ch != '': #vérifie que le bloc soit bien existant
            try:
                out.append(str(TextFromWeb[int(ch,16)])) #retourne le caractère qui correspond à la position en hexa
            except:
                error = True
                out.append('?')
    if error:
        print("Erreur lors du déchiffrement !")
    return ''.join(out)

def GenSum(file_bytes): #génère un hash en sha256 pour la vérification des fichiers appelé sum
    return hashlib.sha256(file_bytes).hexdigest()

def BytesToBase(binary_data, alphabet, occurlist):
    len_bin = len(binary_data)
    #on transforme les bytes en tableau d'integer de taille chunk_size
    splitted_bin=[int.from_bytes(binary_data[ct:ct+chunk_size], byteorder='little') for ct in range(0, len_bin, chunk_size)]
    #On converti le tableau d'integer en base personnalisée afin d'éconmiser de l'espace
    base_data = IntToBase(splitted_bin, alphabet, occurlist, threshold)
    return base_data

def BaseToBytes(base_data , alphabet, occurlist):#inverse de FileToBase
    try:
        int_data = BaseToInt(base_data, alphabet, occurlist, threshold) #base perso vers hexa
        #on écrit le fichier en bytes converti auparavant compressé
        return b''.join([int(i).to_bytes(chunk_size, byteorder='little') for i in int_data]).rstrip(b'\x00')
    except:
        return None    #une erreur est survenue

def Compress(data): #compression des données
    comp_data = lzma.compress(data)
    return comp_data

def Decompress(data): #décompression des données
    decomp_data = lzma.decompress(data)
    return decomp_data

def ChiffrData(binary_data, lists): #chiffre le fichier
    liste_char, liste_pos = lists #liste des caractères présents sur le site et toutes les occurences
    base = BytesToBase(binary_data, liste_char, liste_pos) #lis le fichier et le converti en base personalisée
    chfr = Chiffrer(base, lists) #on chiffre la base (le texte)
    if len(chfr) > 1024:
        compr_tx = Compress(chfr.encode()) #on compresse le texte chiffré
        return compr_tx
    else:
        return chfr

def DeChiffrData(encrypted_data, textfromweb, lists): #dechiffre le fichier
    liste_char,liste_pos = lists #liste des caractères présents sur le site et toutes les occurences
    if isinstance(encrypted_data, bytes): #si c'est des bytes
        decompress_tx = Decompress(encrypted_data).decode()      #on décompresse et converti les bytes en texte
    else:
        decompress_tx = encrypted_data
    dechTx = Dechiffrer(decompress_tx, textfromweb)         #on déchiffre
    return BaseToBytes(dechTx, liste_char, liste_pos)  #on retourne la base transformée en texte



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''    Usage:

    >>> from NimingCypher import NCrypter
    >>> crypter = NCrypter("https://key.com")
    >>> encrypted_str = crypter.crypt_text("simple string")
    >>> print(encrypted_str)
'''

from NimingCypher.Chiffrement import ChiffrData, DeChiffrData, GenSum, Dechiffrer, Chiffrer
from NimingCypher.GetWebText import getsite, tri_char
from os import startfile

#Toutes les explications sont dans les fichiers correspondants
#==================================================================================================================================
class NCrypter:

    def __init__(self, clef):
        self.result = self.setkey(clef)

    def setkey(self, clef):
        self._key = clef #_key pour rendre le paramètre 'key' privé
        try:
            self.textfromweb = getsite(self._key)
            self.charlist = tri_char(self.textfromweb)
            #Initialisation terminée !
            return True
        except:
            #Erreur de connexion !
            return False

    def crypt_file(self, path):
        file = open(path,"rb")
        filebytes = file.read()
        Sum = GenSum(filebytes.rstrip(b'\x00'))#supprime les bytes nuls de fin de ligne
        fl = ChiffrData(filebytes, self.charlist)
        file.close()
        del filebytes
        return (Sum, fl) #hash sha256 et le fichier chiffré

    def decrypt_file(self, textencrypte, path, open_end=False):
        clear_bytes = DeChiffrData(textencrypte, self.textfromweb, self.charlist)
        if clear_bytes != None:
            sf = open(path, "wb+")
            sf.write(clear_bytes)
            sf.close()
            if open_end == True:
                startfile(path)     #ouvre le fichier
            Sum = GenSum(clear_bytes)
            del clear_bytes
            return Sum
        else:
            return "Erreur lors du déchiffrement !"

    def get_security_level(self):
        c = 0
        for l in self.charlist[1]:
            c += len(l)
        security_lvl = int(c/len(self.charlist[1]))
        if security_lvl < 120:
            return "Poor security : %s" % security_lvl
        elif security_lvl >= 120 and security_lvl < 300:
            return "Medium security : %s" % security_lvl
        elif security_lvl >= 300 and security_lvl < 600:
            return "Pretty good security : %s" % security_lvl
        elif security_lvl >= 600 and security_lvl < 1000:
            return "High security : %s" % security_lvl
        elif security_lvl >= 1000 and security_lvl < 2000:
            return "Verry high security : %s" % security_lvl
        elif security_lvl >= 2000:
            return "Top level security : %s" % security_lvl

    def crypt_text(self, text):
        return ChiffrData(text.encode(), self.charlist)

    def decrypt_text(self, text_bytes):
        return DeChiffrData(text_bytes, self.textfromweb, self.charlist)

    def key_hash(self):
        return self.crypt_text(GenSum(self.textfromweb.encode()))

    def check_hash(self, encrypted_hash):
        return GenSum(self.textfromweb.encode()).encode() == self.decrypt_text(encrypted_hash)

#==================================================================================================================================



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Ce module permet de chiffrer des chaines de caractères et des fichier.
Il permet une grande sécurité, et rapidité pour les conversations instantanées.
'''

__all__ = ["NCrypter", "NExchange"]
from NimingCypher.Niming import NCrypter
from NimingCypher.Exchange import NExchange
